import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Insets;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.StringTokenizer;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JTable;

/**

 Class: FilePreview
 Description: Used for previewing the file before actually
         operning it in TerpSpreadSheet. When either the
         the Open or SaveAs dialogs are called, a preiview
         window will appear, if there is no file being
         highlighted, the TerpSpreadSheet logo will appear.
         If a file is selected, then a preview of the first
         10x10 cells will appear.
 
* @author JJ dESi4
* @version 3.0
**************************************************/


public class FilePreview extends JComponent
                          implements PropertyChangeListener
{
	
	/**
		 * Thumbnail Icon
		 */
		ImageIcon thumbnail = null;
		/** Selected File */
		File selectedFile = null;
			/** Buffered Image */
				BufferedImage largeImage = null;
		/** The table */
			JTable table;
		/** The file chooser */
			JFileChooser chooser = null;
		/** Type */
			int type = 0;
	

	/**
		 * Creates new FilePreview instance
		 *
		 * @param fc The JFileChooser to which is belongs
		 * @param file The actual file being highlighted within the dailog
		 */
        public FilePreview(JFileChooser fc, File file)
	{
                setPreferredSize(new Dimension(200, 200));
                fc.addPropertyChangeListener(this);
                type = fc.getDialogType();
		
                chooser = fc;
                
                selectedFile = file;
		//openTableModel();
		//getImage(false);
		

    	}

	/**
	 * Creates new FilePreview instance
	 *
	 * @param fc The JFileChooser to which is belongs
	 *
	 */
        public FilePreview(JFileChooser fc)
        {
            setPreferredSize(new Dimension(200, 200));
            fc.addPropertyChangeListener(this);
            
            chooser = fc;
            
            largeImage = null;
            
        }
	/**
			 * This actually loads the image of the prview or
			 * in case when a directory is chose, the SpreadSheet logo
			 */
	public void loadImage()
	{

            //Don't use createImageIcon (which is a wrapper for getResource)
            //because the image we're trying to load is probably not one
            //of this program's own resources.
            if(largeImage != null)
            {
                ImageIcon tmpIcon = new ImageIcon(largeImage);
                if (tmpIcon != null)
                {
                    if (tmpIcon.getIconWidth() > 90)
                    {
                        thumbnail = new ImageIcon(tmpIcon.getImage().
                                          getScaledInstance(200, 200,
                                                      Image.SCALE_DEFAULT));
                    } 
                    else
                    { 
                	    //no need to miniaturize
                        thumbnail = tmpIcon;
                    }
                }
            }
            else
            {
                ImageIcon tmpIcon = new ImageIcon("TerpSpreadsheet_Splash.jpg");
                
                if(tmpIcon.getIconWidth() > 90)
                    thumbnail = new ImageIcon(tmpIcon.getImage().
                                          getScaledInstance(200, 200,
                                                      Image.SCALE_DEFAULT));
                else
                     thumbnail = tmpIcon;
            }
    	}
	/**
		 * Used to listen for changes within JFileChooser, performs
		 * different actions if a file is chosen, or directory is chosen
		 * or changed
		 *
		 * @param e The property which changed or changed to.
		 */
	public void propertyChange(PropertyChangeEvent e)
	{
        boolean update = false;
        String prop = e.getPropertyName();

        //If the directory changed, don't show an image.
        if (JFileChooser.DIRECTORY_CHANGED_PROPERTY.equals(prop))
	  {
            
            update = true;

          //If a file became selected, find out which one.
        } else if (JFileChooser.SELECTED_FILE_CHANGED_PROPERTY.equals(prop)) 
	  	   {
                       
            		selectedFile = (File) e.getNewValue();
		            update = true;
	         }
       
        //Update the preview accordingly.
        if (update)
	{
            thumbnail = null;
            if (isShowing())
            {
                
                if(chooser.getDialogType() == JFileChooser.SAVE_DIALOG)
                {
                    
                    ImageIcon tmpIcon = new ImageIcon("TerpSpreadsheet_Splash.jpg");
                
                    if(tmpIcon.getIconWidth() > 90)
                        thumbnail = new ImageIcon(tmpIcon.getImage().
                                              getScaledInstance(200, 200,
                                                      Image.SCALE_DEFAULT));
                    else
                         thumbnail = tmpIcon;
                    repaint();
                }
                else
                {
                    try
                    {
                        if(selectedFile.isDirectory())
                        {
                            ImageIcon tmpIcon = new ImageIcon("TerpSpreadsheet_Splash.jpg");
                
                            if(tmpIcon.getIconWidth() > 90)
                                thumbnail = new ImageIcon(tmpIcon.getImage().
                                              getScaledInstance(200, 200,
                                                      Image.SCALE_DEFAULT));
                            else
                                 thumbnail = tmpIcon;
                            repaint();
                        }
                        else
                        {
                            openTableModel();
                            getImage(false);
                            loadImage();
                            repaint();
                        }
                    }
                    catch (Exception what)
                    {
                        ImageIcon tmpIcon = new ImageIcon("TerpSpreadsheet_Splash.jpg");
                
                            if(tmpIcon.getIconWidth() > 90)
                                thumbnail = new ImageIcon(tmpIcon.getImage().
                                              getScaledInstance(200, 200,
                                                      Image.SCALE_DEFAULT));
                            else
                                 thumbnail = tmpIcon;
                            repaint();
                    }
                }
                                 
            }
        }
    }

	/**
	 * Does the graphical paint.
	 *
	 * @param g The graphic to paint.
	 *
	 */

	protected void paintComponent(Graphics g)
	{
            if (thumbnail == null)
                loadImage();
            if (thumbnail != null)
            {
                int x = getWidth()/2 - thumbnail.getIconWidth()/2;
                int y = getHeight()/2 - thumbnail.getIconHeight()/2;

                if (y < 0)
            	     y = 0;
           

                if (x < 5)
                    x = 5;
           
                thumbnail.paintIcon(this, g, x, y);
            }
    	}


	/**
			 * This function opens the data to be read in, stores it within
			 * a local table so that the image can be processed using
			 * the dataset.
			 */
	public void openTableModel()
	{
            StringBuffer textBuf = new StringBuffer();
            String line;
            String grid[][] = new String [51][51];
            String columns[] = new String [51];
            
            //System.out.println("before");
            System.out.println(selectedFile);
            //System.out.println("after");
            try {
                BufferedReader m_in =  new BufferedReader(new FileReader(selectedFile));
                
                
                while ((line = m_in.readLine()) != null) 
                {
                    textBuf.append(line);
                    textBuf.append("\n");
                }   
         
                
                
                //should have entire text of raw data
                String text = textBuf.toString();
                String temp;
            
                StringTokenizer strToken = new StringTokenizer(text, "/");
            
               
                for(int row = 0; row < 51; row++)
                {
                    columns[row] = "NA";
                    for(int col = 0; col < 51; col++)
                    {
                        //gets rid of 255 stuff
                        temp = strToken.nextToken();
                        //gets rid of 0-0-0
                        temp = strToken.nextToken();
                        //gets rid of font
                        temp = strToken.nextToken();
                        //gets rid of numbers after font
                        temp = strToken.nextToken();
                        //gets actual data for cell
                        temp = strToken.nextToken();
                    
                                       
                            
                        grid[row][col] = temp;
                    }   
                    
                }
                
                
                table = new JTable(grid, columns);
            
                //System.out.println((String)table.getValueAt(0,0) + "****");
                //System.out.println((String)table.getValueAt(1,1) + "****");
                
                /* --------DEBUG statements ---------
                System.out.println("gets here");
                for(int i = 0; i < 51; i++)
                {
                    for(int x = 0; x < 51; x++)
                        //System.out.print((String)table.getValueAt(i, x) + "-");
                        System.out.print(grid[i][x] + "-");
                    System.out.println(" ");
                }
                
               System.out.println("but not here");
                 */
                
            }
            catch (FileNotFoundException noFile) {System.out.println(noFile);}
            catch (IOException io) {System.out.println(io);}
            catch (Exception e) {System.out.println(e);}
            
        
    }
		
	/*
	private void fileOpError(String op, String error) {
        UMOptionPane.showMessageDialog(m_tableModel, error, op,
        JOptionPane.ERROR_MESSAGE, null);
        m_tableModel.requestFocus();
    }
	*/
	/**
		 * After the table has been loaded, this function goes through
		 * the table and makes an image out of the first 10x10 cells for
		 * the actual preview image
		 *
		 * @param forPrint Used as a boolean to set colors (for print).
		 */

	public void getImage(boolean forPrint)
	{        
        //JTable table = m_tableModel.getTable();
        int curX = 0;
        int curY = 0;
        int h = 0;
        int w = 0;
        int totalH = 0;
        int totalW = 0;
        for (int i = 1; i < 10; i++)
            totalH += 12;
        for (int j = 1; j < 10; j++)
            totalW += 40;
        
        BufferedImage img = new BufferedImage(totalW + 1, totalH + 1,
        BufferedImage.TYPE_INT_RGB);
        Graphics2D g = img.createGraphics();
        g.setColor(Color.white);
        g.fillRect(0,0, totalW + 1, totalH + 1);
        for (int i = 0; i < 10; i++)
	  {
            h = 12;
            for (int j = 0; j < 10; j++)
		{
                
                    String valueAt = new String();
                w = 40;
                if(selectedFile != null)
                {
                      valueAt = (String)table.getValueAt(i, j);
                }
                else
                {
                     valueAt = new String(" ");
                }
                //UMCell cell = m_tableModel.getCellAt(i,j);
                Color fColor = Color.BLACK;
                Color bColor = Color.WHITE;
                Font font = new Font("Default", Font.PLAIN, 8);
                
                              
                g.setColor(bColor);
                g.fillRect(curX, curY, w, h);
                g.setColor(fColor);
                g.setFont(font);
                g.drawString(valueAt, curX + 3, curY + h - 3);
                LinesBorder b = new LinesBorder(Color.BLACK);
                if (!forPrint)
		    {
                    g.setColor(Color.black);
                    g.drawLine(curX, curY, curX, curY + h);
                    g.drawLine(curX, curY, curX + w, curY);
                    g.drawLine(curX, curY + h, curX + w, curY + h);
                    g.drawLine(curX + w , curY, curX + w, curY + h);
                }
                if (b != null)
		    {
                    g.setColor(Color.black);
                    Insets ins = b.getBorderInsets(null);
                    if (ins.top == 2)
			  {
                        //g.drawLine(curX, curY, curX + w, curY);
                        g.drawLine(curX , curY + 1,
                        curX + w , curY + 1);
                    }
                    if (ins.bottom == 2)
			  {
                        //g.drawLine(curX, curY + h, curX + w, curY + h);
                        g.drawLine(curX , curY + h - 1,
                        curX + w, curY + h - 1);
                    }
                    if (ins.right == 2)
			  {
                        //g.drawLine(curX + w , curY, curX + w, curY + h);
                        g.drawLine(curX + w - 1, curY,
                        curX + w - 1 , curY + h);
                    }
                    if (ins.left == 2)
			  {
                        //g.drawLine(curX, curY, curX, curY + h);
                        g.drawLine(curX + 1, curY,
                        curX + 1, curY + h);
                    }
                    
                }                
                curX += w;                
            }
            curY += h;
            curX = 0;
        }
        if (!forPrint)
	  {
            g.setColor(Color.black);
            g.drawLine(totalW, 0, totalW, totalH);
            g.drawLine(0,totalH,totalW, totalH);
        }        
        //p.setViewState("CPC");
        
        /* Used Only for Testing
        try
        {
            FileOutputStream fos = new FileOutputStream("out.jpg");
            JPEGImageEncoder jpeg = JPEGCodec.createJPEGEncoder(fos);
            jpeg.encode(img);
            fos.close();
        } catch (IOException e){}
        */
        
	largeImage = img;
	
    }


}


