import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.image.BufferedImage;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JTable;

import com.jrefinery.chart.ChartFactory;
import com.jrefinery.chart.JFreeChart;
import com.jrefinery.data.AbstractDataset;
import com.jrefinery.data.XYSeries;
import com.jrefinery.data.XYSeriesCollection;

/**
 * The User Defined Graph class, here the user can define Graph types that then are read
 * into the Spread Sheet application. The user can use any Graph package they want, for example the 
 * Scatter_Graph function uses the JFreeChart package. The user needs to follow the followin format:
 *  1. The graph function should be 'public static void'
 * 	2. The function will only recive the UMSpreadSheet object
 * 	3. Once the function has the SpreadSheet all the graphing and data manipulations are up to the user
 * 	4. There can be as many private helper functions as needed.
 * 
 * @author TSS
 * @version 3.0
 * @see #Scatter_Graph
 */
public class UserGraphs {
  
    /**
     * The Jtable that will be derived form the UMSpreadSheet passed in
     */
    static private JTable table;
    /**
     * The table model that will be derived form the UMSpreadSheet passed in
     */
    static private UMTableModel tablemodel;
    /**
     * The Actual SpreadSheet passed in
     */
    static private TerpSpreadSheet m_spreadsheet;
    
    
    
    
    /**
     * Example of the User Defined Graph, this will create a Scatter Graph
     * The data has to be in 2 adjacent columns and if the user wants, 
     * they can modify the axies and title by putting the following into the selected area:
     *   Column 1	Column 2  
     *   Title		The title User wants
     * 	 X			The X label
     *   Y			The Y label
     * 	 data		data
     *   ...
     * 
     * If no titles are specified there are defaults. 
     * @param spreadsheet - the SpreadSheet object
     */
    public static void Scatter_Graph(TerpSpreadSheet spreadsheet) //throws Exception
    {
        m_spreadsheet = spreadsheet;
        table = spreadsheet.getTable();  //Get the table 
        tablemodel = spreadsheet.getTableModel();  //Get the table model
  
  		//Dataset declarations
  		XYSeries xySeries=null;
  		XYSeriesCollection dataset=null;
  		System.out.println( "XY_GRAPH selected" );
  	
  		//Default names
  		String tbl_name=new String("Graph");
  		String x_name=new String("X");
  		String y_name=new String("Y");
   
   		if( table.getSelectedRowCount() != 0 && table.getSelectedColumnCount() > 1 ) //Must have at least 2 columns selected
        {
        	//Get the start row/colum indecies and the last row/col 
            int startRow = table.getSelectedRow();
            int startCol = table.getSelectedColumn();
            int numrows = table.getSelectedRowCount() ;
            int numcols = table.getSelectedColumnCount();
            int endRow = startRow + numrows;
            int endCol = startCol + numcols;
  			  
  	       	boolean error = false;
 		   	dataset = new XYSeriesCollection();
           	xySeries = new XYSeries("Scatter");
        
        	//Go through the selected area cell by cell
       	 	for ( int row = startRow ; row < endRow; row++ )
        	{
	    		//Get the cells for each row
	            UMCell XCell = tablemodel.getCellAt( row , startCol );
    	        UMCell YCell = tablemodel.getCellAt( row , startCol + 1 );
            
            
            if ( XCell.getType() == UMCell.TEXT ) //If TEXT check for titles
            {
                if(XCell.getValue().toString().compareTo("Title")==0)
                { tbl_name=new String (YCell.getValue().toString());
                }
				else  if(XCell.getValue().toString().compareTo("X")==0)
                { x_name=new String (YCell.getValue().toString());
                }
				else  if(XCell.getValue().toString().compareTo("Y")==0)
                { y_name=new String (YCell.getValue().toString());
                }
            }
            //Else add the Numbers to the Datasets
            else if ( XCell.getType() != UMCell.TEXT && YCell.getType() != UMCell.TEXT ) //make sure they are valid cells
            {
                xySeries.add( (Number)XCell.getValue() , (Number)YCell.getValue()  );
            }
        }
         dataset =new XYSeriesCollection( xySeries );
  	}
  
    AbstractDataset Dataset = null;
          
    //Makes a call to the function which draws the graph in a frame.
    if ( dataset != null )
    {
      	//com.jrefinery.chart.plot.PlotOrientation	orient;
        JFreeChart chart = ChartFactory.createScatterPlot( tbl_name, x_name ,y_name , 
                                                                       ( XYSeriesCollection )dataset , true, true, true );
        outputGraph( chart );
    }
 }
     
  
     //Takes a JFreeChart and outputs it in a frame
    /**
     * Helper function for the Scatter_Graph. Takes a JFreeChart and outputs it in a frame.
     * @param chart - The chart to draw
     */
    private static void outputGraph( JFreeChart chart ) //throws Exception
    {
            // write chart as JPEG file
            //ChartUtilities.saveChartAsJPEG(new File("chart.jpg"), chart, 500,300);

            final BufferedImage image = chart.createBufferedImage(500,300);

            JLabel lblChart = new JLabel();
            lblChart.setHorizontalAlignment(JLabel.CENTER);
            lblChart.setVerticalAlignment(JLabel.CENTER);
            ImageIcon icon = new ImageIcon( image );
            lblChart.setIcon( icon );

            JFrame frame = new JFrame( "University of Maryland CMSC 435 Spreadsheet " );
            
            JMenuBar menu = new JMenuBar();
            JMenu copy = new JMenu( "Copy" );
            
            
            /*JMenuItem copyToClipboard = new JMenuItem( "Copy to Clipboard" );
            copyToClipboard.addActionListener( new ActionListener() {
                public void actionPerformed( ActionEvent evt ) {
                    copyToClipboard( image );
                }});
     
            copy.add( copyToClipboard );    
            menu.add( copy );    
            frame.setJMenuBar( menu );
*/
            frame.getContentPane().setLayout(new BorderLayout());
            frame.getContentPane().add(lblChart, BorderLayout.CENTER);
            frame.setSize(600,400);
            frame.setVisible(true);
   
            
    }
    
    /**
     * Copies the graph image to the Clipboard
     * @param image - Image of the graph
     */
    private static void copyToClipboard( BufferedImage image )
    {
        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        
        Image graphImage;
        graphImage = (Image)image;
        
        UMClipObj clip = new UMClipObj(graphImage);
        
        //clipboard.setContents(clip, this);
    }
    
    /*
    public void lostOwnership(Clipboard clipboard, Transferable transferable) {
    }*/
   
   
    
    
} // UserFunctions

